SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

//RasterizerState gRS_NoCulling
//{
//	CullMode = NONE;
//};

DepthStencilState gDSS_LessEqual
{
	DepthFunc = LESS_EQUAL;
};

TextureCube m_CubeMap : CubeMap;

cbuffer cbChangesEveryFrame
{
	matrix matWorldViewProj : WorldViewProjection;
}

struct VS_IN
{
	float3 posL : POSITION;
};

struct VS_OUT
{
	float4 posH : SV_POSITION;
	float3 texC : TEXCOORD;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
VS_OUT VS( VS_IN vIn )
{
	VS_OUT vOut = (VS_OUT)0;
	
	// set z = w so that z/w = 1 (i.e., skydome always on far plane).
	// use local vertex position as cubemap lookup vector
	vOut.posH = mul (float4(vIn.posL, 0.0f), matWorldViewProj).xyww;
	vOut.texC = vIn.posL;

	return vOut;
}
//--------------------------------------------------------------------------------------
// Pixel XMeshShader
//--------------------------------------------------------------------------------------
float4 PS( VS_OUT pIn): SV_Target
{
	float4 diffuseColor = m_CubeMap.Sample( samLinear,pIn.texC )*0.1f;
	float4 atCol = float4(0.3f, 0.3f, 0.65f, 1);
	float atDotNeg  = dot(normalize(pIn.texC), float3(0, -1, 0));
	float atDotPos  = saturate(-atDotNeg);
	if(atDotNeg>0)
	{
		diffuseColor = atCol*atDotNeg;
	}
	float atStrength = pow(1-atDotPos, 15);
	diffuseColor += lerp(diffuseColor, atCol, atStrength);
	return float4(diffuseColor.rgb, 1);
}

technique10 Render
{
    pass P0
    {
		//SetRasterizerState(gRS_NoCulling);
		SetDepthStencilState(gDSS_LessEqual, 0);
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}