//DX10 - FLAT SHADER
//Digital Arts & Entertainment

//AtmosphericReentryShader.fx
//author: Robert Lindner
//class: 2DAE1


//GLOBAL VARIABLES
//****************
float4x4 gWorldViewProj : WORLDVIEWPROJECTION;
float4x4 gWorld: WORLD;
float4x4 gViewProj: WORLD;
float4x4 gViewInverse: VIEWINVERSE;
float4x4 gLightViewProj;
float4x4 gMoveViewProj;

//Enables
bool gUseDiffuse = false;
bool gUseSpecular = false;
bool gUseNormal = false;
bool gInvertGreen = false;
bool gUseEnvironment = false;
bool gUseFresnel = false;
// Textures
textureCUBE gTextureEnv;
Texture2D gDiffuseMap;
Texture2D gNormalMap;
Texture2D gSpecularMap;

Texture2D gShadowMap;
Texture2D gMoveShadowMap;
Texture2D gNoiseMap;
// Light and Shadow
float3 gLightDirection : DIRECTION = float3(-.577f, -.577f, .577f);
float gShadowMapBias =0.0008f;
uint2 gShadowMapSize = uint2(1920, 1080);
float gShadowStrength = 1;
float gSunStrength = 0.8f;
//Atmospheric
float4 gAtmosphereColor = float4(0.5f, 0.65f, 0.9f, 1);
float3 gMoveDir = float3(0, -1, 0);
float gMoveVel = 4000;
float gReentryStrength = 1;
float4 gReentryLightColor = float4(1, 0.5f, 0.2f, 1);
float4 gReentryLightColorRed = float4(1, 0.15f, 0, 0.00f);
float4 gReentryLightColorBright = float4(1, 0.85f, 0.3f, 0.05f);
float gMoveShadowBias = 0.003f; 
//Animated length
int gSubImageX = 0;
int gSubImageY = 0;
int gSubImageChannel = 0;
int gSubImageXN = 0;
int gSubImageYN = 0;
int gSubImageChannelN = 0;
float gNoiseLerp = 0;
//Geometry parameters
float gVelMultiplier = 0.00006f;
float gLengthVar = 75;
float gIncrement = 0.25f;
float gTaperSize = 0.5f; 
//Second pass parameters
float gBounceOffset = 1;
float4 gBounceColor = float4(0.5f, 0.5f, 0.99f, 0.1f);
float gBounceColLerpOffset = 0.5f;
//Ambient parameters
float4 gColorAmbient : COLOR = float4(0.05, 0.05, 0.05, 1.0);
float gAmbientIntensity = 1.0f;
//Diffuse parameters
float4 gColorDiffuse : COLOR = float4(1.0, 1.0, 1.0, 1.0);
//Specular parameters
float4 gColorSpecular : COLOR = float4(1.0, 1.0, 1.0, 1.0);
float gShininess = 50.0f;
//emvironment parameters
float gReflectionStrength = 1.0f;
//cubemap parameters
float gFresnelPower = 1.0f;
float gFresnelMultiplier = 1.0f;
float gFresnelHardness = 1.0f;
float4 gFresnelColor : COLOR = float4(1.0f,1.0f,1.0f,1.0f);
//States 
RasterizerState gRS_NoCulling
{
	CullMode = NONE;
};
BlendState AlphaBlending 
{     
	BlendEnable[0] = TRUE;
	SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
	BlendOp = ADD;
	SrcBlendAlpha = ZERO;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0f;
};
DepthStencilState DisableDepthWriting
{
	DepthEnable = TRUE;
	DepthWriteMask = ZERO;
};
RasterizerState BackCulling
{
	CullMode = BACK;
};
//Texture Samplers
SamplerState gSamplerEnvMap
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
	AddressV = WRAP;
	AddressW = WRAP;
};
SamplerState gDiffuseSampler
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
};
SamplerState gNormalSampler
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = WRAP;
	AddressV = WRAP;
};
SamplerComparisonState cmpSampler
{
   Filter = COMPARISON_MIN_MAG_MIP_LINEAR;
   AddressU = MIRROR;
   AddressV = MIRROR;
   ComparisonFunc = LESS_EQUAL;
};
//Input and output structires

struct VS_INPUT
{
    float3 Position : POSITION;
    float3 Normal : NORMAL;
	float3 Tangent : TANGENT;
	float2 TexCoord : TEXCOORD0;
};
struct VS_OUTPUT
{
    float4 Position : SV_POSITION;
    float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;
    float3 WorldPosition : TEXCOORD1;
	float4 lpos : TEXCOORD2;
	float4 MoveLightPos : TEXCOORD3;
    float3 Tangent : TENGENT;
};

//Helper functions
//*****************
float2 texOffset( int u, int v )
{
    return float2( u * 1.0f/gShadowMapSize.x, v * 1.0f/gShadowMapSize.y );
}
float3 CalculateNormal(float3 n, float3 t, float2 texCoord)
{
	float3 normal = normalize(n);
	if(gUseNormal)
	{
		float3 tangent = normalize(t);
		float3 binormal = normalize(cross(tangent, normal));
		float3x3 localAxis = float3x3(tangent, binormal, normal);
		float3 sampledNormal = ((gNormalMap.Sample( gNormalSampler,texCoord ))*2)-float3(1, 1, 1);
		if(gInvertGreen)sampledNormal.y *= -1;
		normal = normalize(mul(normalize(sampledNormal), (float3x3)localAxis));
	}
	return normal;
}
float CalculateShadow(float4 lightPos, Texture2D shadowMap, float bias)
{
	float4 lpos = float4(lightPos.xyz/lightPos.w, lightPos.w);
    if( lpos.x < -1.0f || lpos.x > 1.0f ||
        lpos.y < -1.0f || lpos.y > 1.0f ||
        lpos.z < 0.0f  || lpos.z > 1.0f ) 
	{
		return 1;
	}
	else
	{
		lpos.x = lpos.x/2 + 0.5;
		lpos.y = lpos.y/-2 + 0.5;
		lpos.z -= bias;
		float sum;
		float x, y;
		for (y = -2; y <=2; y += 1.0f)
		{
			for (x = -2; x <= 2; x += 1.0f)
			{
				sum += shadowMap.SampleCmpLevelZero(cmpSampler, lpos.xy + texOffset(x,y), lpos.z );
			}
		}
		float shadowFactor = sum / 25.0f;
		return saturate(shadowFactor + 1 - gShadowStrength);
	}
}
float3 CalculateSkyboxColor(float3 vec)
{
	float3 environment = gReflectionStrength * gTextureEnv.Sample(gSamplerEnvMap,vec);
	float4 gAtmosphereColor = float4(0.5f, 0.5f, 0.9f, 1);
	float atDotNeg  = dot(vec, float3(0, -1, 0));
	float atDotPos  = saturate(-atDotNeg);
	if(atDotNeg>0)
	{
		environment = gAtmosphereColor*atDotNeg;
	}
	float atStrength = pow(1-atDotPos, 15);
	environment += lerp(environment, gAtmosphereColor, atStrength);
	return environment;
}
float3 CalculateFresnel(float3 normal, float3 viewDirection)
{
	float fresnel = 1 - saturate(abs(dot(normal,viewDirection)));
	fresnel = pow(fresnel,gFresnelPower);
	fresnel *= gFresnelMultiplier;
	fresnel *= pow((1 - saturate(dot(float3(0,-1,0),normal))),gFresnelHardness); //Mask
	return fresnel * gFresnelColor * gUseFresnel;
}
float3 CalculateLighting(float3 normal, float3 lDir, float3 viewDirection, float2 texCoord)
{
	//Diffuse
	float diffLightVal = max(dot(-normal, lDir), 0);
	float3 diffuse = gColorDiffuse.rgb*diffLightVal;
	if(gUseDiffuse)diffuse*=gDiffuseMap.Sample(gDiffuseSampler, texCoord);
	
	
	//Specular
	float3 halfVec = -normalize(viewDirection+lDir);
	float specStrength = pow(max(dot(normal, halfVec), 0), gShininess);
	float3 specular = gColorSpecular.rgb*specStrength;
	if(gUseSpecular)specular*=gSpecularMap.Sample(gDiffuseSampler, texCoord);
	
	return diffuse+specular;
}
float3 CalculatePlanetShine(float3 normal, float3 lDir, float3 atmoCol, float2 texCoord)
{
	//Diffuse
	float diffLightVal = dot(-normal, lDir);
	if(diffLightVal>=0)diffLightVal=pow(diffLightVal, 5);
	else diffLightVal=-pow(-diffLightVal, 0.5f);
	diffLightVal = (diffLightVal+1)*0.5f;
	float3 diffuse = gColorDiffuse.rgb*diffLightVal;
	if(gUseDiffuse)diffuse*=gDiffuseMap.Sample(gDiffuseSampler, texCoord);
	return diffuse*atmoCol;
}
//Normal Pass
//******************
VS_OUTPUT MainVS(VS_INPUT input)
{
    VS_OUTPUT output = (VS_OUTPUT)0;
    output.Position = mul(float4(input.Position,1.0f),gWorldViewProj);
   
    output.Normal = mul(input.Normal,(float3x3)gWorld);
	output.TexCoord = input.TexCoord;
	output.Tangent = mul(normalize(input.Tangent), (float3x3)gWorld);
   
    // Use a float4 for transforming
    output.WorldPosition = mul(float4(input.Position,1.0f),gWorld);
	
    output.lpos = mul(float4(input.Position, 1), mul(gWorld, gLightViewProj));
    output.MoveLightPos = mul(float4(input.Position, 1), mul(gWorld, gMoveViewProj));
	
	
    return output;
}
float4 MainPSBlinn(VS_OUTPUT input) : SV_TARGET 
{
	float3 normal = CalculateNormal(input.Normal, input.Tangent, input.TexCoord);
    float3 viewDirection = normalize(input.WorldPosition.xyz - gViewInverse[3].xyz);
	float3 reflectedVector = normalize(reflect(viewDirection, normal));
	//Environment
	float3 environment = float3(gUseEnvironment, gUseEnvironment, gUseEnvironment);
	environment *= CalculateSkyboxColor(reflectedVector);
	environment *= CalculateFresnel(normal, viewDirection);
	//Lighting
	float shadowFactor = CalculateShadow(input.lpos, gShadowMap, gShadowMapBias);
	float moveShadow = CalculateShadow(input.MoveLightPos, gMoveShadowMap, gMoveShadowBias);
	
	float3 ambient = gColorAmbient*gAmbientIntensity;
	float3 lightingSun = CalculateLighting(normal, gLightDirection, viewDirection, input.TexCoord)*gSunStrength;
	float3 planetShine = CalculatePlanetShine(normal, float3(0, 1, 0), gAtmosphereColor.rgb*0.1f, input.TexCoord);
	float3 reentryLighting = CalculateLighting(normal, gMoveDir, viewDirection, input.TexCoord)*gReentryLightColor.rgb*gReentryLightColor.a;
    return float4(environment+ambient+shadowFactor*lightingSun+planetShine+reentryLighting*moveShadow*gReentryStrength*gMoveVel/4000, 1);
}


//Effect Pass
//******************
//General
struct GS_DATA
{
	float4 Position : SV_POSITION;
	float4 Color : COLOR;
};
void CreateVertex(inout TriangleStream<GS_DATA> triStream, float3 pos, float4 col)
{
	GS_DATA geomData = (GS_DATA) 0;
    geomData.Position = mul( float4(pos,1.0f), gViewProj);
	geomData.Color = col;
	triStream.Append(geomData);
}
float4 EffectPSBlinn(GS_DATA input) : SV_TARGET 
{
    return input.Color;
}
//Pass two
struct GS_INPUT
{
	float3 Position : POSITION;
	float3 CenterVec : POSITION2;
	float3 Normal : NORMAL;
	float4 LightPosition : TEXCOORD1;
	float2 NoiseTC : TEXCOORD2;
	float2 NoiseTCN : TEXCOORD3;
};
GS_INPUT EffectVS(VS_INPUT input)
{
    GS_INPUT output = (GS_INPUT)0;
   
	output.CenterVec = input.Position;
    output.Position = mul(float4(input.Position,1.0f), gWorld);//float4(input.Position,1.0f);
    output.Normal = mul(input.Normal,(float3x3)gWorld);
    output.LightPosition = mul(float4(input.Position, 1), mul(gWorld, gMoveViewProj));
	output.NoiseTC = float2(gSubImageX*0.25f+input.TexCoord.x*0.25f, gSubImageY*0.25f+input.TexCoord.y*0.25f);
	output.NoiseTCN = float2(gSubImageXN*0.25f+input.TexCoord.x*0.25f, gSubImageYN*0.25f+input.TexCoord.y*0.25f);
	
    return output;
}
float GetNoiseLength(float2 tc, float2 tcN)
{
	float lengthVar = gNoiseMap.SampleLevel(gDiffuseSampler, tc, 0)[gSubImageChannel];
	float lengthVarNext = gNoiseMap.SampleLevel(gDiffuseSampler, tcN, 0)[gSubImageChannelN];
	return lerp(lengthVarNext, lengthVar, gNoiseLerp);
}
[maxvertexcount(6)]
void EffectGS(triangle GS_INPUT vertex[3][1], inout TriangleStream<GS_DATA> triStream)
{
	//Handle animated noise length lookup, store in float3
	float lengthGeneral = gMoveVel*gVelMultiplier;
	float3 lengthVar = float3(lengthGeneral, lengthGeneral, lengthGeneral);
	for(int i = 0; i < 3; i++) lengthVar[i] += GetNoiseLength(vertex[i][0].NoiseTC, vertex[i][0].NoiseTCN)*lengthGeneral*gLengthVar;
	
	//movement "Shadow" occlusion
	float3 occlusion = float3(CalculateShadow(vertex[0][0].LightPosition, gMoveShadowMap, gMoveShadowBias),
								CalculateShadow(vertex[1][0].LightPosition, gMoveShadowMap, gMoveShadowBias),
								CalculateShadow(vertex[2][0].LightPosition, gMoveShadowMap, gMoveShadowBias));
	lengthVar*=occlusion;
	
	//colors
	float strength = gMoveVel/4000;
	float3 brightCol = lerp(gReentryLightColorBright.rgb, gBounceColor.rgb, saturate(strength-gBounceColLerpOffset));
	float4 col1 = float4(brightCol, gReentryLightColorBright.a)*strength; 
	float4 col2 = lerp(gReentryLightColorBright, gReentryLightColorRed, gIncrement)*strength;  
	float4 col3 = gReentryLightColorRed*strength; 
	
	float curve = gIncrement*1.5f;                                
	float3 nMove = normalize(gMoveDir);   
	
	for(int i = 0; i < 3; i++)
	{           
		float3 normal = normalize(vertex[i][0].Normal);    
		float velDot = -dot(normal, nMove);   
		//determining length based on normal
		if(velDot >= 0 && occlusion[i]>0.9f)
		{              
			int j = (i+1)%3;   
			
			float3 nCenter = normalize(vertex[i][0].CenterVec);  
			float3 nSize = normalize(cross(nMove, nCenter));
			
			velDot = pow(1 - velDot, 3);//assuming velDot larger 0
			float3 length_i = lengthVar[i]*velDot*nMove;
			float3 length_j = lengthVar[j]*velDot*nMove;      
			      
			float3 outward = -cross(nMove, nSize)*gIncrement*lengthVar[i]*velDot;	
			float3 sizeEnd = nSize*gTaperSize;
			float3 sizeTop = nSize*lerp(1, gTaperSize, gIncrement);
			
			float3 end_j =  sizeEnd	+length_j			 + (outward);
			float3 end_i = -sizeEnd	+length_i			 + (outward);
			float3 top_j =  sizeTop	+length_j*gIncrement + (outward*curve);
			float3 top_i = -sizeTop	+length_i*gIncrement + (outward*curve);
			//Create Geometry (Trianglestrip) alternating between this and its adjacent edge
			CreateVertex(triStream, vertex[i][0].Position,  		col1);
			CreateVertex(triStream, vertex[j][0].Position, 			col1);
			CreateVertex(triStream, vertex[i][0].Position + top_i, 	col2);
			CreateVertex(triStream, vertex[j][0].Position + top_j,	col2);
			CreateVertex(triStream, vertex[i][0].Position + end_i, 	col3);
			CreateVertex(triStream, vertex[j][0].Position + end_j, 	col3);
		}
	}
}
//Pass three
struct GS_INPUT2
{
	float3 Position : POSITION;
	float3 Normal : NORMAL;
	float4 LightPosition : TEXCOORD1;
};
GS_INPUT2 EffectVS2(VS_INPUT input)
{
    GS_INPUT2 output = (GS_INPUT2)0;
   
    output.Position = mul(float4(input.Position,1.0f), gWorld).rgb;
    output.Normal = mul(input.Normal,(float3x3)gWorld);
    output.LightPosition = mul(float4(input.Position, 1), mul(gWorld, gMoveViewProj));
	
    return output;
}
[maxvertexcount(3)]
void EffectGS2(triangle GS_INPUT2 vertex[3][1], inout TriangleStream<GS_DATA> triStream)
{
	float strength = gMoveVel/4000;
	float3 occlusion = float3(CalculateShadow(vertex[0][0].LightPosition, gMoveShadowMap, gMoveShadowBias),
								CalculateShadow(vertex[1][0].LightPosition, gMoveShadowMap, gMoveShadowBias),
								CalculateShadow(vertex[2][0].LightPosition, gMoveShadowMap, gMoveShadowBias));
	                       
	float3 nMove = normalize(gMoveDir);   

	for(int i = 0; i < 3; i++)
	{   
		float3 normal = normalize(vertex[i][0].Normal);    
		float velDot = dot(normal, nMove); 
		velDot = pow(velDot, 3);
		if(occlusion[i]>0.9f && velDot>0)    
		{  
			
			float alpha = -velDot*occlusion[i]*strength;;
			float4 col = gBounceColor*alpha;
			float3 offset = alpha * gBounceOffset * normal;
			
			CreateVertex(triStream, vertex[i][0].Position+offset, col);
		}
	}
}

//TECHNIQUES
//**********
technique11 TechBlinn 
{
	pass p0 
	{
		SetRasterizerState(gRS_NoCulling);	
		SetVertexShader(CompileShader(vs_4_0, MainVS()));
		SetPixelShader(CompileShader(ps_4_0, MainPSBlinn()));
	}
	pass p1
	{
		SetRasterizerState(gRS_NoCulling);	      
		SetDepthStencilState(DisableDepthWriting, 0);
        SetBlendState(AlphaBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
		
		SetVertexShader(CompileShader(vs_4_0, EffectVS()));
		SetGeometryShader(CompileShader(gs_4_0, EffectGS()));
		SetPixelShader(CompileShader(ps_4_0, EffectPSBlinn()));
	}
	pass p2
	{
		SetRasterizerState(gRS_NoCulling);	      
		SetDepthStencilState(DisableDepthWriting, 0);
        SetBlendState(AlphaBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
		
		SetVertexShader(CompileShader(vs_4_0, EffectVS2()));
		SetGeometryShader(CompileShader(gs_4_0, EffectGS2()));
		SetPixelShader(CompileShader(ps_4_0, EffectPSBlinn()));
	}
}