//=============================================================================
//// Shader uses position and texture
//=============================================================================
SamplerState samPoint
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Mirror;
    AddressV = Mirror;
};

Texture2D gTexture;
float gExposure = 1.0f;
float gGamma = 2.2f;
float4 gThreshold = float4(1, 1, 1, 1);

/// Create Depth Stencil State (ENABLE DEPTH WRITING)
DepthStencilState DepthStencilOn
{
	//DepthWriteMask = 1;
	DepthEnable = TRUE;
};

/// Create Rasterizer State (Backface culling) 
RasterizerState BackfaceCulling
{
	CullMode = BACK;
};


//IN/OUT STRUCTS
//--------------
struct VS_INPUT
{
    float3 Position : POSITION;
	float2 TexCoord : TEXCOORD0;

};

struct PS_INPUT
{
    float4 Position : SV_POSITION;
	float2 TexCoord : TEXCOORD1;
};


//VERTEX SHADER
//-------------
PS_INPUT VS(VS_INPUT input)
{
	PS_INPUT output = (PS_INPUT)0;
	// Set the Position
	output.Position = float4(input.Position, 1);
	// Set the TexCoord
	output.TexCoord = input.TexCoord;
	
	return output;
}


//PIXEL SHADER
//------------
float4 PS(PS_INPUT input): SV_Target
{	
	float width; float height; gTexture.GetDimensions(width, height);
	float dx = 1.0f/width;
	float dy = 1.0f/height;
	float3 acc;
	for(int x = -2; x < 3; x++)
	{
		for(int y = -2; y < 3; y++)
		{
			float2 tc = float2(x*dx*2, y*dy*2)+input.TexCoord;
			float4 sample = gTexture.Sample( samPoint, tc );
			acc+=float3(max(sample.r-gThreshold.r, 0),
						max(sample.g-gThreshold.g, 0),
						max(sample.b-gThreshold.b, 0));
		}
	}
	acc/=25;
	float3 inCol = gTexture.Sample( samPoint,input.TexCoord ).rgb;
	inCol+=acc;
	
	// Reinhard tone mapping
	float3 mapped = float3(1, 1, 1) - 
	float3(exp(-inCol.x * gExposure), exp(-inCol.y * gExposure), exp(-inCol.z * gExposure));
	// Gamma correction 
	mapped = pow(mapped, 1.0f / gGamma);
	
	return float4(saturate(mapped), 1);
}


//TECHNIQUE
//---------
technique11 Blur
{
    pass P0
    {
		SetRasterizerState(BackfaceCulling);
		SetDepthStencilState(DepthStencilOn, 0);
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}