//DX10 - FLAT SHADER
//Digital Arts & Entertainment

//UberShader.fx
//author: Robert Lindner
//class: 2DAE1


//GLOBAL VARIABLES
//****************
float4x4 gWorldViewProj : WORLDVIEWPROJECTION;
float4x4 gWorld: WORLD;
float4x4 gViewInverse: VIEWINVERSE;
float4x4 gLightViewProj;

//Enables
bool gUseDiffuse = false;
bool gUseSpecular = false;
bool gUseNormal = false;
bool gInvertGreen = false;
bool gUseOpacity = false;
bool gUseEnvironment = false;
bool gUseFresnel = false;
// Textures
textureCUBE gTextureEnv;
Texture2D gDiffuseMap;
Texture2D gNormalMap;
Texture2D gSpecularMap;
Texture2D gOpacityMap;

Texture2D gShadowMap;
// Light and Shadow
float3 gLightDirection : DIRECTION = float3(-.577f, -.577f, .577f);
float gShadowMapBias =0.005f;
uint2 gShadowMapSize = uint2(1920, 1080);
float gShadowStrength = 1;
//Ambient parameters
float4 gColorAmbient : COLOR = float4(0.05, 0.05, 0.05, 1.0);
float gAmbientIntensity = 1.0f;
//Diffuse parameters
float4 gColorDiffuse : COLOR = float4(1.0, 1.0, 1.0, 1.0);
//Specular parameters
float4 gColorSpecular : COLOR = float4(1.0, 1.0, 1.0, 1.0);
float gShininess = 50.0f;
//emvironment parameters
float gReflectionStrength = 1.0f;
float gRefractionStrength = 0.0f;
float gIOR = 0.0f;
//cubemap parameters
float gFresnelPower = 1.0f;
float gFresnelMultiplier = 1.0f;
float gFresnelHardness = 1.0f;
float4 gFresnelColor : COLOR = float4(1.0f,1.0f,1.0f,1.0f);
 //Opacity parameters
float gOpacity = 1.0f;
//States 
BlendState EnableBlending
{
    BlendEnable[0] = TRUE;
    DestBlend = INV_SRC_ALPHA;
    SrcBlend = SRC_ALPHA;
};
RasterizerState gRS_NoCulling
{
	CullMode = NONE;
};
SamplerState gSamplerEnvMap
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
	AddressV = WRAP;
	AddressW = WRAP;
};
SamplerState gDiffuseSampler
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = WRAP;
	AddressV = WRAP;
};
SamplerState gNormalSampler
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = WRAP;
	AddressV = WRAP;
};
SamplerComparisonState cmpSampler
{
   // sampler state
   Filter = COMPARISON_MIN_MAG_MIP_LINEAR;
   AddressU = MIRROR;
   AddressV = MIRROR;
 
   // sampler comparison state
   ComparisonFunc = LESS_EQUAL;
};
//Input and output structires

struct VS_INPUT
{
    float3 Position : POSITION;
    float3 Normal : NORMAL;
	float3 Tangent : TANGENT;
	float2 TexCoord : TEXCOORD0;
};
struct VS_OUTPUT
{
    float4 Position : SV_POSITION;
    float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;
    float3 WorldPosition : TEXCOORD1;
	float4 lpos : TEXCOORD2;
    float3 Tangent : TENGENT;
};

//MAIN VERTEX SHADER
//******************
VS_OUTPUT MainVS(VS_INPUT input)
{
    VS_OUTPUT output = (VS_OUTPUT)0;
    output.Position = mul(float4(input.Position,1.0f),gWorldViewProj);
   
    output.Normal = mul(input.Normal,(float3x3)gWorld);
	output.TexCoord = input.TexCoord;
	output.Tangent = mul(normalize(input.Tangent), (float3x3)gWorld);
   
    // Use a float4 for transforming
    output.WorldPosition = mul(float4(input.Position,1.0f),gWorld);
	
    output.lpos = mul(float4(input.Position, 1), mul(gWorld, gLightViewProj));
	
    return output;
}

//MAIN PIXEL SHADER
//*****************
float2 texOffset( int u, int v )
{
    return float2( u * 1.0f/gShadowMapSize.x, v * 1.0f/gShadowMapSize.y );
}
float4 MainPSBlinn(VS_OUTPUT input) : SV_TARGET 
{
	//Calculate normal
	float3 normal = normalize(input.Normal);
	if(gUseNormal)
	{
		float3 tangent = normalize(input.Tangent);
		float3 binormal = normalize(cross(tangent, normal));
		float3x3 localAxis = float3x3(tangent, binormal, normal);
		float3 sampledNormal = ((gNormalMap.Sample( gNormalSampler,input.TexCoord ))*2)-float3(1, 1, 1);
		if(gInvertGreen)sampledNormal.y *= -1;
		normal = normalize(mul(normalize(sampledNormal), (float3x3)localAxis));
	}
	
	//Context
    float3 viewDirection = normalize(input.WorldPosition.xyz - gViewInverse[3].xyz);
	
	//Environment
	float3 environment = float3(0, 0, 0);
	if(gUseEnvironment)
	{
		float3 reflectedVector = reflect(viewDirection, normal);
		reflectedVector = normalize(reflectedVector);
		float3 refractedVector = refract(viewDirection, normal, gIOR);
		refractedVector = normalize(refractedVector);
		
		environment = 
			gReflectionStrength * gTextureEnv.Sample(gSamplerEnvMap,reflectedVector) +
			gRefractionStrength * gTextureEnv.Sample(gSamplerEnvMap,refractedVector);
		
		//fresnel
		if(gUseFresnel)
		{
			float fresnel = 1 - saturate(abs(dot(normal,viewDirection)));
			fresnel = pow(fresnel,gFresnelPower);
			fresnel *= gFresnelMultiplier;
			fresnel *= pow((1 - saturate(dot(float3(0,-1,0),normal))),gFresnelHardness); //Mask
			environment *= fresnel * gFresnelColor;
		}
	}
	
	//Ambient
	float3 ambient = gColorAmbient*gAmbientIntensity;
	
	//Opacity
    float opacity = gOpacity;
	if(gUseOpacity)opacity*=gOpacityMap.Sample(gDiffuseSampler, input.TexCoord);
    
	//Shadow
	float4 lpos = input.lpos;
    lpos.xyz /= lpos.w;
	float shadowFactor;
    if( lpos.x < -1.0f || lpos.x > 1.0f ||
        lpos.y < -1.0f || lpos.y > 1.0f ||
        lpos.z < 0.0f  || lpos.z > 1.0f ) 
	{
		shadowFactor = 1;
	}
	else
	{
		lpos.x = lpos.x/2 + 0.5;
		lpos.y = lpos.y/-2 + 0.5;
		lpos.z -= gShadowMapBias;
		float sum;
		float x, y;
		for (y = -2; y <=2; y += 1.0f)
		{
			for (x = -2; x <= 2; x += 1.0f)
			{
				sum += gShadowMap.SampleCmpLevelZero(cmpSampler, lpos.xy + texOffset(x,y), lpos.z );
			}
		}
		shadowFactor = sum / 25.0f;
		shadowFactor = saturate(shadowFactor + 1 - gShadowStrength);
	}
	
	//Diffuse
	float diffLightVal = max(dot(-normal, gLightDirection), 0);
	float3 diffuse = gColorDiffuse.rgb*diffLightVal;
	if(gUseDiffuse)diffuse*=gDiffuseMap.Sample(gDiffuseSampler, input.TexCoord);
	
	//Specular
	float3 halfVec = -normalize(viewDirection+gLightDirection);
	float specStrength = pow(max(dot(normal, halfVec), 0), gShininess);
	float3 specular = gColorSpecular.rgb*specStrength;
	if(gUseSpecular)specular*=gSpecularMap.Sample(gDiffuseSampler, input.TexCoord);
	
    float3 finalColor = environment+ambient+shadowFactor*(diffuse+specular);
	
    return float4(finalColor, opacity);
}
float4 MainPSPhong(VS_OUTPUT input) : SV_TARGET 
{
	//Calculate normal
	float3 normal = normalize(input.Normal);
	if(gUseNormal)
	{
		float3 tangent = normalize(input.Tangent);
		float3 binormal = normalize(cross(tangent, normal));
		float3x3 localAxis = float3x3(tangent, binormal, normal);
		float3 sampledNormal = ((gNormalMap.Sample( gNormalSampler,input.TexCoord ))*2)-float3(1, 1, 1);
		if(gInvertGreen)sampledNormal.y *= -1;
		normal = normalize(mul(normalize(sampledNormal), (float3x3)localAxis));
	}
	
	//Context
    float3 viewDirection = normalize(input.WorldPosition.xyz - gViewInverse[3].xyz);
	float3 reflectedVector = reflect(viewDirection, normal);
	reflectedVector = normalize(reflectedVector);
	
	//Environment
	float3 environment = float3(0, 0, 0);
	if(gUseEnvironment)
	{
		float3 refractedVector = refract(viewDirection, normal, gIOR);
		refractedVector = normalize(refractedVector);
		
		environment = 
			gReflectionStrength * gTextureEnv.Sample(gSamplerEnvMap,reflectedVector) +
			gRefractionStrength * gTextureEnv.Sample(gSamplerEnvMap,refractedVector);
		
		//fresnel
		if(gUseFresnel)
		{
			float fresnel = 1 - saturate(abs(dot(normal,viewDirection)));
			fresnel = pow(fresnel,gFresnelPower);
			fresnel *= gFresnelMultiplier;
			fresnel *= pow((1 - saturate(dot(float3(0,-1,0),normal))),gFresnelHardness); //Mask
			environment *= fresnel * gFresnelColor;
		}
	}
	
	//Ambient
	float3 ambient = gColorAmbient*gAmbientIntensity;
	
	//Diffuse
	float diffLightVal = max(dot(-normal, gLightDirection), 0);
	float3 diffuse = gColorDiffuse.rgb*diffLightVal;
	if(gUseDiffuse)diffuse*=gDiffuseMap.Sample(gDiffuseSampler, input.TexCoord);
	
	//Specular
	float specStrength = pow(max(dot(-gLightDirection, reflectedVector), 0), gShininess);
	float3 specular = gColorSpecular.rgb*specStrength;
	if(gUseSpecular)specular*=gSpecularMap.Sample(gDiffuseSampler, input.TexCoord);
	
	//Opacity
    float opacity = gOpacity;
	if(gUseOpacity)opacity*=gOpacityMap.Sample(gDiffuseSampler, input.TexCoord);
	
    float3 finalColor = environment+ambient+diffuse+specular;
	
    return float4(finalColor, opacity);
}

//TECHNIQUES
//**********
technique11 TechBlinn 
{
	pass p0 
	{
		SetRasterizerState(gRS_NoCulling);	
		SetVertexShader(CompileShader(vs_4_0, MainVS()));
		SetPixelShader(CompileShader(ps_4_0, MainPSBlinn()));
	}
}
technique11 TechPhong 
{
	pass p0 
	{
		SetRasterizerState(gRS_NoCulling);	
		SetVertexShader(CompileShader(vs_4_0, MainVS()));
		SetPixelShader(CompileShader(ps_4_0, MainPSPhong()));
	}
}
technique11 TechBlinnBlending 
{
	pass p0 
	{
		SetBlendState(EnableBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
		SetRasterizerState(gRS_NoCulling);	
		SetVertexShader(CompileShader(vs_4_0, MainVS()));
		SetPixelShader(CompileShader(ps_4_0, MainPSBlinn()));
	}
}
technique11 TechPhongBlending 
{
	pass p0 
	{
		SetBlendState(EnableBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
		SetRasterizerState(gRS_NoCulling);	
		SetVertexShader(CompileShader(vs_4_0, MainVS()));
		SetPixelShader(CompileShader(ps_4_0, MainPSPhong()));
	}
}